import 'package:comunic/forez/helpers/forez_group_helper.dart';
import 'package:comunic/forez/ui/routes/search_users.dart';
import 'package:comunic/helpers/groups_helper.dart';
import 'package:comunic/helpers/users_helper.dart';
import 'package:comunic/lists/group_members_list.dart';
import 'package:comunic/lists/users_list.dart';
import 'package:comunic/models/group.dart';
import 'package:comunic/models/group_membership.dart';
import 'package:comunic/models/user.dart';
import 'package:comunic/ui/widgets/account_image_widget.dart';
import 'package:comunic/ui/widgets/async_screen_widget.dart';
import 'package:comunic/utils/account_utils.dart';
import 'package:comunic/utils/conversations_utils.dart';
import 'package:comunic/utils/intl_utils.dart';
import 'package:comunic/utils/log_utils.dart';
import 'package:comunic/utils/ui_utils.dart';
import 'package:flutter/material.dart';

/// Forez directory screen
///
/// @author Pierre Hubert

enum _PopupMenuActions {
  PRIVATE_CONVERSATION,
  CANCEL_INVITATION,
  ACCEPT_REQUEST,
  REJECT_REQUEST
}

class ForezDirectoryScreen extends StatefulWidget {
  @override
  _ForezDirectoryScreenState createState() => _ForezDirectoryScreenState();
}

class _ForezDirectoryScreenState extends State<ForezDirectoryScreen> {
  final _key = GlobalKey<AsyncScreenWidgetState>();

  UsersList _users;
  GroupMembersList _members;

  Future<void> _load() async {
    _members = await GroupsHelper.getMembersList(forezGroup.id);
    _users = await UsersHelper().getListWithThrow(_members.usersID);
  }

  @override
  Widget build(BuildContext context) => Stack(
        children: [
          AsyncScreenWidget(
            onReload: _load,
            onBuild: onBuild,
            errorMessage: tr("Failed to load members list!"),
            key: _key,
          ),
          Positioned(
            child: FloatingActionButton(
              onPressed: _doSearch,
              child: Icon(Icons.search),
            ),
            right: 20,
            bottom: 20,
          ),
        ],
      );

  Widget onBuild() => ListView.builder(
        itemBuilder: (c, i) => _ForezMemberTile(
          member: _members[i],
          user: _users.getUser(_members[i].userID),
          selectedAction: _doPopupMenuAction,
        ),
        itemCount: _members.length,
      );

  void _doPopupMenuAction(_PopupMenuActions action, User user) async {
    try {
      switch (action) {
        case _PopupMenuActions.PRIVATE_CONVERSATION:
          openPrivateConversation(context, user.id);
          break;

        case _PopupMenuActions.CANCEL_INVITATION:
          await GroupsHelper.cancelInvitation(forezGroup.id, user.id);
          _key.currentState.refresh();
          break;

        case _PopupMenuActions.ACCEPT_REQUEST:
          await GroupsHelper.respondRequest(forezGroup.id, user.id, true);
          _key.currentState.refresh();
          break;

        case _PopupMenuActions.REJECT_REQUEST:
          await GroupsHelper.respondRequest(forezGroup.id, user.id, false);
          _key.currentState.refresh();
          break;
      }
    } catch (e, s) {
      logError(e, s);
      snack(context, tr("Error while processing action!"));
    }
  }

  void _doSearch() async {
    final user = await searchUser(context, UsersList()..addAll(_users));
    print("found ${user?.fullName}");
    //if (user != null) _openUserProfile(user);
  }
}

class _ForezMemberTile extends StatelessWidget {
  final User user;
  final GroupMembership member;
  final Function(_PopupMenuActions, User) selectedAction;

  const _ForezMemberTile({
    Key key,
    @required this.user,
    @required this.member,
    @required this.selectedAction,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) => ListTile(
        leading: AccountImageWidget(user: user),
        title: Text(user.fullName),
        subtitle: Text(member.membershipText),
        trailing: member.level == GroupMembershipLevel.PENDING
            ? _buildRequestedButton()
            : _buildPopupMenuButton(),
      );

  Widget _buildPopupMenuButton() => user.id == userID()
      ? null
      : PopupMenuButton(
          itemBuilder: (c) => popupMenuItems,
          onSelected: (a) => selectedAction(a, user),
        );

  Widget _buildRequestedButton() => IntrinsicWidth(
        child: Row(
          children: [
            ElevatedButton(
              onPressed: () =>
                  selectedAction(_PopupMenuActions.ACCEPT_REQUEST, user),
              child: Icon(Icons.check),
              style: ButtonStyle(
                  visualDensity: VisualDensity.compact,
                  backgroundColor: MaterialStateProperty.all(Colors.green)),
            ),
            SizedBox(width: 5),
            OutlinedButton(
              onPressed: () =>
                  selectedAction(_PopupMenuActions.REJECT_REQUEST, user),
              child: Icon(Icons.close),
              style: ButtonStyle(
                  visualDensity: VisualDensity.compact,
                  foregroundColor: MaterialStateProperty.all(Colors.redAccent)),
            ),
          ],
        ),
      );

  List<PopupMenuItem<_PopupMenuActions>> get popupMenuItems => [
        PopupMenuItem(
          child: Text(tr("Private conversation")),
          value: _PopupMenuActions.PRIVATE_CONVERSATION,
        )
      ]..addAll(member.level == GroupMembershipLevel.INVITED &&
              forezGroup.isAtLeastModerator
          ? [
              PopupMenuItem(
                child: Text(tr("Cancel invitation")),
                value: _PopupMenuActions.CANCEL_INVITATION,
              ),
            ]
          : []);
}
